@@ -33,15 +33,16 @@ class CloudSat(Product):
 
     def xrange(self, layer, level):
         w = self.profile['zoom'][level]['width']
-        time = self.swath['Profile_time']
         start = self.swath.attributes['start_time']
+        time = self.swath['Profile_time'][:].astype(np.float64)
         start = dt.datetime.strptime(start, "%Y%m%d%H%M%S").replace(tzinfo=pytz.utc)
-        t_origin = self.profile['origin'][0]
-        t1 = self._dt2ms(self._time(time[0], start) - t_origin) + self.offset()
-        t2 = self._dt2ms(self._time(time[-1], start) - t_origin) + self.offset()
-        x1 = int(math.floor(t1 / w))
-        x2 = int(math.floor(t2 / w))
-        return list(range(x1, x2+1))
+        origin = self.profile['origin'][0]
+        origin_to_start = (start - origin).total_seconds()
+        t = (origin_to_start + time) * 1000 + self.offset()
+        quality = self.swath['Data_quality'][:]
+        mask = (quality & 0x40) == 0
+        x = np.unique((t[mask] / w).astype(int))
+        return x.tolist()
 
     def zrange(self, layer, level):
         # One-dimensional layer.
@@ -108,16 +109,6 @@ class CloudSat(Product):
         n2 = (t2 - t0)/sampling_interval
         n1_ = ccbrowse.utils.coerce(int(math.floor(n1)), n0, nn-1)
         n2_ = ccbrowse.utils.coerce(int(math.ceil(n2)+1), n0, nn-1)
-        factor = height.attributes['factor']
-        offset = height.attributes['offset']
-        height = (height[n1_:n2_,:] - offset)/factor
-        height_max = height.max(0)
-        height_min = height.min(0)
-        m1 = len(height_max) - np.searchsorted(height_max[::-1], z2) - 1
-        m1 = ccbrowse.utils.coerce(m1, m0, mm-1)
-        m2 = len(height_min) - np.searchsorted(height_min[::-1], z1) + 1
-        m2 = ccbrowse.utils.coerce(m2, m0, mm-1)
-        height = height[:,m1:m2]
 
         # Trajectory - special case.
         if layer in ('trajectory', self.NAME+'-trajectory'):
@@ -150,6 +141,17 @@ class CloudSat(Product):
                                      raw_data).astype(np.float32).reshape(1, 256)
             return tile
 
+        factor = height.attributes['factor']
+        offset = height.attributes['offset']
+        height = (height[n1_:n2_,:] - offset)/factor
+        height_max = height.max(0)
+        height_min = height.min(0)
+        m1 = len(height_max) - np.searchsorted(height_max[::-1], z2) - 1
+        m1 = ccbrowse.utils.coerce(m1, m0, mm-1)
+        m2 = len(height_min) - np.searchsorted(height_min[::-1], z1) + 1
+        m2 = ccbrowse.utils.coerce(m2, m0, mm-1)
+        height = height[:,m1:m2]
+
         # Two-dimensional layer.
         raw_data = dataset[n1_:n2_,m1:m2].astype(np.float32)
 
